/* typemaps.i --- guile-specific typemaps -*- c -*-
   Copyright (C) 2000 Matthias Koeppe <mkoeppe@mail.math.uni-magdeburg.de>

   $Header: /cvs/projects/SWIG/Lib/guile/typemaps.i,v 1.17 2000/09/21 21:06:17 mkoeppe Exp $  */

/* Unlike other SWIG language modules, the Guile module handles all
   non-pointer types uniformly via typemaps. Here are the
   definitions.

   The SIMPLE_MAP macro below defines the whole set of typemaps needed
   for simple types. */

%define SIMPLE_MAP(C_NAME, SCM_TO_C, C_TO_SCM, SCM_NAME)
 %typemap (guile, in)          C_NAME "$target = SCM_TO_C($source);";
 %typemap (guile, varin)       C_NAME "$target = SCM_TO_C($source);";
 %typemap (guile, out)         C_NAME "$target = C_TO_SCM($source);";
 %typemap (guile, varout)      C_NAME "$target = C_TO_SCM($source);";
 %typemap (guile, indoc)       C_NAME "($arg <SCM_NAME>)";
 %typemap (guile, varindoc)    C_NAME "($arg <SCM_NAME>)";
 %typemap (guile, outdoc)      C_NAME "<SCM_NAME>";
 %typemap (guile, varoutdoc)   C_NAME "<SCM_NAME>";
 %typemap (guile, in)          C_NAME *INPUT (C_NAME temp)
 "temp = (C_NAME) C_TO_SCM($source); $target = &temp;"
 %typemap (guile, indoc)       C_NAME *INPUT "($arg <SCM_NAME>)";
 %typemap (guile, ignore)      C_NAME *OUTPUT (C_NAME temp)
 "$target = &temp;"
 %typemap (guile, argout)      C_NAME *OUTPUT
 "GUILE_APPEND_RESULT(C_TO_SCM(*$target));";
 %typemap (guile, argoutdoc)   C_NAME *OUTPUT "($arg <SCM_NAME>)";
 %typemap (guile, in)          C_NAME *BOTH = C_NAME *INPUT;
 %typemap (guile, indoc)       C_NAME *BOTH = C_NAME *INPUT;
 %typemap (guile, argout)      C_NAME *BOTH = C_NAME *OUTPUT;
 %typemap (guile, argoutdoc)   C_NAME *BOTH = C_NAME *OUTPUT;
%enddef
 
 SIMPLE_MAP(bool, gh_scm2bool, gh_bool2scm, boolean);
 SIMPLE_MAP(char, GSWIG_scm2char, gh_char2scm, char);
 SIMPLE_MAP(unsigned char, GSWIG_scm2char, gh_char2scm, char);
 SIMPLE_MAP(int, gh_scm2int, gh_int2scm, integer);
 SIMPLE_MAP(short, gh_scm2int, gh_int2scm, integer);
 SIMPLE_MAP(long, gh_scm2long, gh_long2scm, integer);
 SIMPLE_MAP(ptrdiff_t, gh_scm2long, gh_long2scm, integer);
 SIMPLE_MAP(unsigned int, gh_scm2ulong, gh_ulong2scm, integer);
 SIMPLE_MAP(unsigned short, gh_scm2ulong, gh_ulong2scm, integer);
 SIMPLE_MAP(unsigned long, gh_scm2ulong, gh_ulong2scm, integer);
 SIMPLE_MAP(size_t, gh_scm2ulong, gh_ulong2scm, integer);
 SIMPLE_MAP(float, gh_scm2double, gh_double2scm, real);
 SIMPLE_MAP(double, gh_scm2double, gh_double2scm, real);
 SIMPLE_MAP(char *, GSWIG_scm2str, gh_str02scm, string);
 SIMPLE_MAP(const char *, GSWIG_scm2str, gh_str02scm, string);




%define SIMPLE_MAP_STR(C_NAME, SCM_TO_C, C_TO_SCM, SCM_NAME)
 %typemap (guile, in)          C_NAME { char* x; x=SCM_TO_C($source); $target = new string(x); scm_must_free(x);};
 %typemap (guile, varin)       C_NAME { char* x; x=SCM_TO_C($source); $target = new string(x); scm_must_free(x);};
 %typemap (guile, out)         C_NAME "$target = C_TO_SCM($source->c_str());";
 %typemap (guile, varout)      C_NAME "$target = C_TO_SCM($source->c_str());";
%enddef
 
SIMPLE_MAP_STR(string , GSWIG_scm2str, gh_str02scm, string);
SIMPLE_MAP_STR(string&, GSWIG_scm2str, gh_str02scm, string);
SIMPLE_MAP_STR(const string , GSWIG_scm2str, gh_str02scm, string);
SIMPLE_MAP_STR(const string&, GSWIG_scm2str, gh_str02scm, string);

%typemap (guile, freearg) string, string&, const string, const string& "if ($target) delete $target;";




%typemap (guile, in) vector<string> (vector<string> temp) { 

  SCM v = gh_list_to_vector( $source );
  unsigned long len = gh_vector_length (v);
  unsigned long i=0;

  $target = new vector<string>();

  for( i=0; i<len; i++) 
    {
      //      cerr << " copying i:" << i << endl;
      SCM scm_i = gh_ulong2scm(i);
      SCM scm_str = gh_vector_ref( v, scm_i );
      char* x = GSWIG_scm2str(scm_str); 
      $target->push_back(string(x));
      scm_must_free(x);
    }

};
%typemap (guile, freearg) vector<string> "if ($target) delete $target;";


%typemap (guile, out) vector<string> {

  int i=0;
  int count = $source->size();

  cerr << " convert vector<> to scheme world" << endl;

  if( !count ) 
    {
      $target = gh_list( SCM_UNDEFINED );
    }
  else
    {
      const string& s = (*$source)[count-1];
      cerr << " convert vector<> to scheme world 2... s: " << s << endl;
      $target = gh_list( gh_str02scm( s.c_str()),  SCM_UNDEFINED );
      cerr << " convert vector<> to scheme world 3 " << endl;

      for( i=count-2; i>=0; i-- )
	{
	  cerr << " convert vector<> to scheme world i:" << i << endl;
	  const string& s = (*$source)[i];
	  $target = gh_cons( gh_str02scm( s.c_str()), $target );
	}
    }
};


%typemap (guile, out) vector<foo*> {

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

  swig_type_info *element_type_mangled_name = SWIGTYPE_p_foo;
  typedef  foo*  element_type;

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

  int i=0;
  int count = $source->size();

  if( !count ) 
    {
      $target = gh_list( SCM_UNDEFINED );
    }
  else 
    {
      element_type s = (*$source)[count-1];  
      SCM s_scm = SWIG_Guile_MakePtr( s, element_type_mangled_name );
      $target = gh_list( s_scm,  SCM_UNDEFINED );

      for( i=count-2; i>=0; i-- )
	{
	  element_type s = (*$source)[i];
	  s_scm = SWIG_Guile_MakePtr( s, element_type_mangled_name );
	  $target = gh_cons( s_scm , $target );
	}
    }

};





%typemap (guile, out) vector<Witme_DirItem*> {

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

  swig_type_info *element_type_mangled_name = SWIGTYPE_p_Witme_DirItem;
  typedef  Witme_DirItem*  element_type;

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

  int i=0;
  int count = $source->size();

  if( !count ) 
    {
      $target = gh_list( SCM_UNDEFINED );
    }
  else 
    {
      element_type s = (*$source)[count-1];  
      SCM s_scm = SWIG_Guile_MakePtr( s, element_type_mangled_name );
      $target = gh_list( s_scm,  SCM_UNDEFINED );

      for( i=count-2; i>=0; i-- )
	{
	  element_type s = (*$source)[i];
	  s_scm = SWIG_Guile_MakePtr( s, element_type_mangled_name );
	  $target = gh_cons( s_scm , $target );
	}
    }

};



/* GSWIG_scm2str makes a malloc'ed copy of the string, so get rid of it after
   the function call. */

%typemap (guile, freearg) char *, const char * "if ($target) scm_must_free($target);";

/* Void */

%typemap (guile, out) void "gswig_result = GH_UNSPECIFIED;";
%typemap (guile, outdoc) void "";

/* typemaps.i ends here */
